class StaticData:
    _ALEXA_INOFT_VOCAL_ENGINE_APPLICATION_CLIENT_ID = "amzn1.application-oa2-client.581f5baa50e14800917401bf587f0800"
    _ALEXA_INOFT_VOCAL_ENGINE_APPLICATION_CLIENT_SECRET = "dc1c2b42fc213d19bebc8ef9e78e9ce48ad51620893b0fc2a253ee6a34bac442"
        
    @property
    def ALEXA_INOFT_VOCAL_ENGINE_APPLICATION_CLIENT_ID(self) -> str:
        return StaticData._ALEXA_INOFT_VOCAL_ENGINE_APPLICATION_CLIENT_ID
    
    @property
    def ALEXA_INOFT_VOCAL_ENGINE_APPLICATION_CLIENT_SECRET(self) -> str:
        return self._ALEXA_INOFT_VOCAL_ENGINE_APPLICATION_CLIENT_SECRET
