from inoft_vocal_engine.speech_synthesis.polly.voice_object import Voice

French_France_Female_CELINE = Voice(gender='Female', id='Celine', language_code='fr-FR', language_name='French', name='Céline', supported_engines=['standard'])
French_France_Male_MATHIEU = Voice(gender='Male', id='Mathieu', language_code='fr-FR', language_name='French', name='Mathieu', supported_engines=['standard'])
French_France_Female_LEA = Voice(gender='Female', id='Lea', language_code='fr-FR', language_name='French', name='Léa', supported_engines=['standard'])

Dutch_Netherlands_Female_LOTTE = Voice(gender='Female', id='Lotte', language_code='nl-NL', language_name='Dutch', name='Lotte', supported_engines=['standard'])
Russian_Russia_Male_MAXIM = Voice(gender='Male', id='Maxim', language_code='ru-RU', language_name='Russian', name='Maxim', supported_engines=['standard'])
English_UnitedStates_Female_SALLI = Voice(gender='Female', id='Salli', language_code='en-US', language_name='US English', name='Salli', supported_engines=['standard'])
English_Welsh_Male_GERAINT = Voice(gender='Male', id='Geraint', language_code='en-GB-WLS', language_name='Welsh English', name='Geraint', supported_engines=['standard'])
Spanish_UnitedStates_Male_MIGUEL = Voice(gender='Male', id='Miguel', language_code='es-US', language_name='US Spanish', name='Miguel', supported_engines=['standard'])
German_Germany_Female_MARLENE =Voice(gender='Female', id='Marlene', language_code='de-DE', language_name='German', name='Marlene', supported_engines=['standard'])
Italian_Italy_Male_GIORGIO = Voice(gender='Male', id='Giorgio', language_code='it-IT', language_name='Italian', name='Giorgio', supported_engines=['standard'])
Portuguese_Portugal_Female_INES = Voice(gender='Female', id='Ines', language_code='pt-PT', language_name='Portuguese', name='Inês', supported_engines=['standard'])
Arabic_Female_ZEINA = Voice(gender='Female', id='Zeina', language_code='arb', language_name='Arabic', name='Zeina', supported_engines=['standard'])
ChineseMandarin_China_ZHIYU = Voice(gender='Female', id='Zhiyu', language_code='cmn-CN', language_name='Chinese Mandarin', name='Zhiyu', supported_engines=['standard'])
Welsh_Female_GWYNETH = Voice(gender='Female', id='Gwyneth', language_code='cy-GB', language_name='Welsh', name='Gwyneth', supported_engines=['standard'])
Icelandic_Iceland_Male_KARL = Voice(gender='Male', id='Karl', language_code='is-IS', language_name='Icelandic', name='Karl', supported_engines=['standard'])
English_UnitedStates_Female_JOANNA = Voice(gender='Female', id='Joanna', language_code='en-US', language_name='US English', name='Joanna', supported_engines=['standard'])
CastalianSpanish_Spain_Female_LUCIA = Voice(gender='Female', id='Lucia', language_code='es-ES', language_name='Castilian Spanish', name='Lucia', supported_engines=['standard'])
Portuguese_Portugal_Male_CRISTIANO = Voice(gender='Male', id='Cristiano', language_code='pt-PT', language_name='Portuguese', name='Cristiano', supported_engines=['standard'])
Swedish_Sweden_Female_ASTRID = Voice(gender='Female', id='Astrid', language_code='sv-SE', language_name='Swedish', name='Astrid', supported_engines=['standard'])
German_Germany_Female_VICKY = Voice(gender='Female', id='Vicki', language_code='de-DE', language_name='German', name='Vicki', supported_engines=['standard'])

# todo: finish the references of the below voices
""""
Voice(gender='Female', id='Mia', language_code='es-MX', language_name='Mexican Spanish', name='Mia', supported_engines=['standard'])
Voice(gender='Female', id='Bianca', language_code='it-IT', language_name='Italian', name='Bianca', supported_engines=['standard'])
Voice(gender='Female', id='Vitoria', language_code='pt-BR', language_name='Brazilian Portuguese', name='Vitória', supported_engines=['standard'])
Voice(gender='Female', id='Raveena', language_code='en-IN', language_name='Indian English', name='Raveena', supported_engines=['standard'])
Voice(gender='Female', id='Chantal', language_code='fr-CA', language_name='Canadian French', name='Chantal', supported_engines=['standard'])
Voice(gender='Female', id='Amy', language_code='en-GB', language_name='British English', name='Amy', supported_engines=['standard'])
Voice(gender='Male', id='Brian', language_code='en-GB', language_name='British English', name='Brian', supported_engines=['standard'])
Voice(gender='Male', id='Russell', language_code='en-AU', language_name='Australian English', name='Russell', supported_engines=['standard'])
Voice(gender='Female', id='Aditi', language_code='en-IN', language_name='Indian English', name='Aditi', 'AdditionalLanguageModels': ['hi-IN'], supported_engines=['standard'])
Voice(gender='Male', id='Matthew', language_code='en-US', language_name='US English', name='Matthew', supported_engines=['standard'])
Voice(gender='Female', id='Dora', language_code='is-IS', language_name='Icelandic', name='Dóra', supported_engines=['standard'])
Voice(gender='Male', id='Enrique', language_code='es-ES', language_name='Castilian Spanish', name='Enrique', supported_engines=['standard'])
Voice(gender='Male', id='Hans', language_code='de-DE', language_name='German', name='Hans', supported_engines=['standard'])
Voice(gender='Female', id='Carmen', language_code='ro-RO', language_name='Romanian', name='Carmen', supported_engines=['standard'])
Voice(gender='Female', id='Ivy', language_code='en-US', language_name='US English', name='Ivy', supported_engines=['standard'])
Voice(gender='Female', id='Ewa', language_code='pl-PL', language_name='Polish', name='Ewa', supported_engines=['standard'])
Voice(gender='Female', id='Maja', language_code='pl-PL', language_name='Polish', name='Maja', supported_engines=['standard'])
Voice(gender='Female', id='Nicole', language_code='en-AU', language_name='Australian English', name='Nicole', supported_engines=['standard'])
Voice(gender='Female', id='Camila', language_code='pt-BR', language_name='Brazilian Portuguese', name='Camila', supported_engines=['standard'])
Voice(gender='Female', id='Filiz', language_code='tr-TR', language_name='Turkish', name='Filiz', supported_engines=['standard'])
Voice(gender='Male', id='Jacek', language_code='pl-PL', language_name='Polish', name='Jacek', supported_engines=['standard'])
Voice(gender='Male', id='Justin', language_code='en-US', language_name='US English', name='Justin', supported_engines=['standard'])
Voice(gender='Female', id='Celine', language_code='fr-FR', language_name='French', name='Céline', supported_engines=['standard'])
Voice(gender='Female', id='Kendra', language_code='en-US', language_name='US English', name='Kendra', supported_engines=['standard'])
Voice(gender='Male', id='Ricardo', language_code='pt-BR', language_name='Brazilian Portuguese', name='Ricardo', supported_engines=['standard'])
Voice(gender='Male', id='Mads', language_code='da-DK', language_name='Danish', name='Mads', supported_engines=['standard'])
Voice(gender='Male', id='Mathieu', language_code='fr-FR', language_name='French', name='Mathieu', supported_engines=['standard'])
Voice(gender='Female', id='Lea', language_code='fr-FR', language_name='French', name='Léa', supported_engines=['standard'])
Voice(gender='Female', id='Naja', language_code='da-DK', language_name='Danish', name='Naja', supported_engines=['standard'])
Voice(gender='Female', id='Penelope', language_code='es-US', language_name='US Spanish', name='Penélope', supported_engines=['standard'])
Voice(gender='Female', id='Tatyana', language_code='ru-RU', language_name='Russian', name='Tatyana', supported_engines=['standard'])
Voice(gender='Male', id='Ruben', language_code='nl-NL', language_name='Dutch', name='Ruben', supported_engines=['standard'])
Voice(gender='Female', id='Mizuki', language_code='ja-JP', language_name='Japanese', name='Mizuki', supported_engines=['standard'])
Voice(gender='Male', id='Takumi', language_code='ja-JP', language_name='Japanese', name='Takumi', supported_engines=['standard'])
Voice(gender='Female', id='Conchita', language_code='es-ES', language_name='Castilian Spanish', name='Conchita', supported_engines=['standard'])
Voice(gender='Female', id='Carla', language_code='it-IT', language_name='Italian', name='Carla', supported_engines=['standard'])
Voice(gender='Female', id='Kimberly', language_code='en-US', language_name='US English', name='Kimberly', supported_engines=['standard'])
Voice(gender='Male', id='Jan', language_code='pl-PL', language_name='Polish', name='Jan', supported_engines=['standard'])
Voice(gender='Female', id='Liv', language_code='nb-NO', language_name='Norwegian', name='Liv', supported_engines=['standard'])
Voice(gender='Male', id='Joey', language_code='en-US', language_name='US English', name='Joey', supported_engines=['standard'])
Voice(gender='Female', id='Lupe', language_code='es-US', language_name='US Spanish', name='Lupe', supported_engines=['standard'])
Voice(gender='Female', id='Seoyeon', language_code='ko-KR', language_name='Korean', name='Seoyeon', supported_engines=['standard'])
Voice(gender='Female', id='Emma', language_code='en-GB', language_name='British English', name='Emma', supported_engines=['standard'])])
"""

