import_bot_from_botpress_to_new_project = "{requester_account_id}:import-data:import-bot-from-botpress-to-new-project"
import_bot_from_botpress_in_existing_project = "{requester_account_id}:{account_project_id}:import-data:import-bot-from-botpress-in-existing-project"
